/*
 * Created by SharpDevelop.
 * User: ivo.stoykov
 * Date: 20.3.2013 .
 * Time: 13:50 .
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace DictNS
{
  /// <summary>
  /// Description of DictUtils.
  /// </summary>
  public static class DictUtils
  {
    
    private static Configuration conf = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);

    public static void _dump(Exception e)
    {
      try
      {
        String filename = String.Format("{0}\\{1}_err.log", Application.StartupPath, Application.ProductName);
        File.AppendAllText(filename, e.StackTrace);
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    /// <summary>
    /// Gets a configuration setting if exists. If does not exists - creates it and return default value or empty string.
    /// </summary>
    /// <param name="key">Key where the value is stored upon</param>
    /// <param name="defaultValue">Default value to be used if key does not exists.</param>
    /// <returns>The value if found, default value, if supplied or empty sing otherwise</returns>
    public static string GetAppSetting(String key, String defaultValue = "")
    {
      if(conf.AppSettings.Settings[key] != null)
      {
        return conf.AppSettings.Settings[key].Value;
      }
      SetAppSetting(key, defaultValue);
      return defaultValue;
    }

    /// <summary>
    /// Add a value to the settings. If key does not exists it will create it.
    /// </summary>
    /// <param name="key">Key that holds the value.</param>
    /// <param name="val">value to be stored with this key.</param>
    public static void SetAppSetting(String key, String val)
    {
      if (conf.AppSettings.Settings[key] == null) { conf.AppSettings.Settings.Add(key, val); }
      else { conf.AppSettings.Settings[key].Value = val; }
    }

    public static void RemoveSetting(String key) { conf.AppSettings.Settings.Remove(key);  }

    /// <summary>
    /// Save application settings or show error message
    /// </summary>
    public static void SaveSettings()
    {
      try
      {
        conf.Save(ConfigurationSaveMode.Full);
        conf = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
      }
      catch (ConfigurationErrorsException e) 
      {
        MessageBox.Show(e.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    public static String GetDefaultSearchEngine()
    {
      String se = DictUtils.GetAppSetting("defaultSE");
      if (String.IsNullOrEmpty(se)) { return String.Empty; }
      int idx = Convert.ToInt32(se);
      String[] items = DictUtils.GetAppSetting("se").Split('@');
      if (idx > items.Count() - 1) { return String.Empty; }
      return items[idx];
    }
  }
}
