﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Reflection;
using System.Security;
using System.Security.Permissions;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace DictNS
{
  public partial class dict : Form
  {

    #region [Private members]
    private enum LogTextType { Info, Sucess, Error }
    private WebClient cli;
    private List<String> userWords;
    private Int32 rtbLinesCounter = 0;
    public struct userProxy
    {
      public String pUrl;
      public Int32 pPort;
      public String pUser;
      public String pPass;
      public String pType;
      public bool pIsUserDefined;
    }
    private userProxy _userDefinedProxy =  new userProxy();
    #endregion

    #region [Constructors]
    public dict()
    {
      InitializeComponent();
      #if DEBUG
      logIt("InitializeComponent passed.");
      #endif
      GetUserProxy();
      #if DEBUG
      logIt("User defined proxy read.");
      #endif
      InitializeWebClient();
      #if DEBUG
      logIt("InitializeWebClient finished.");
      #endif
      InitializeGUI();
      #if DEBUG
      logIt("InitializeGUI finished.");
      #endif
    }

    #endregion

    #region [Initialization Section]
    /// <summary>
    /// Restore path saved into AppSetting during previous session or creates AppSettings if not exists.
    /// Value will be the place where words are saved into from
    /// </summary>
    private void InitializeGUI()
    {
      Version v = Assembly.GetExecutingAssembly().GetName().Version;
      Text = String.Format("{0} - {1}.{2} build {3}", Text, v.Major, v.Minor, v.Build);
      toolStripProgressBar1.Visible = false;
      #if DEBUG
      logIt(String.Format("{0} fired", (new StackTrace(true).GetFrame(0).GetMethod().Name)));
      #endif
      InitializePath();
      InitializeWords();
    }

    /// <summary>
    /// Prepare list with stored words
    /// </summary>
    private async void InitializeWords()
    {
      if(!Directory.Exists(cbPath.Text)) { return;  }
      cbWordsList.DataSource = null;
      List<String> _wordsList = new List<string>();
      DirectoryInfo dir = new System.IO.DirectoryInfo(cbPath.Text);
      FileInfo[] fileList = await Task.Run(() => { return dir.GetFiles("*.mp3", SearchOption.TopDirectoryOnly); }  );
      foreach (FileInfo f in fileList)
      {
        _wordsList.Add(f.Name.Replace(".mp3", "").Replace("_", " "));
      }
      String _words = cbWordsList.Text ?? String.Empty;
      cbWordsList.DataSource = _wordsList;
      cbWordsList.AutoCompleteSource = AutoCompleteSource.ListItems;
      cbWordsList.AutoCompleteMode = AutoCompleteMode.Suggest;
      cbWordsList.SelectedIndex = -1;
      if(!String.IsNullOrEmpty(_words)){  
        cbWordsList.Text = _words;
        cbWordsList.SelectAll();
      }
      cbWordsList.Focus();
    }
    
    /// <summary>
    /// Prepare list with last used storage paths
    /// </summary>
    private void InitializePath()
    {
      if(DictUtils.GetAppSetting("cbPathHistoryAllowed", "true").ToLower().Equals("false")) {  return;  } // history disabled
      String _userPath = DictUtils.GetAppSetting("userPath");
      if(String.IsNullOrEmpty(_userPath)) { return; }
      List<String> _pathList = _userPath.Split(';').ToList<String>();
      Int32 _activeUserPath = 0;
      Int32.TryParse(DictUtils.GetAppSetting("activeUserPath", "0"), out _activeUserPath);
      cbPath.DataSource = _pathList;
      cbPath.AutoCompleteSource = AutoCompleteSource.ListItems;
      cbPath.AutoCompleteMode = AutoCompleteMode.Suggest;
      cbPath.SelectedIndex = _activeUserPath < 0 ? 0 : _activeUserPath;
    }
    
    /// <summary>
    /// Prepare WebClient for web comunication
    /// </summary>
    private void InitializeWebClient()
    {
      #if DEBUG
      logIt(String.Format("{0} fired", (new StackTrace(true).GetFrame(0).GetMethod().Name)));
      #endif
      cli = new WebClient();
      if (cli == null) { throw new ApplicationException(String.Format("{0}: Failed to create WebClient!", (new StackTrace(true).GetFrame(0).GetMethod().Name))); }
      cli.DownloadDataCompleted += WebClient_DownloadDataCompleted;
      cli.DownloadFileCompleted += WebClient_DownloadFileCompleted;
      cli.DownloadProgressChanged += webClient_ProgressChanged;
      //cli.Credentials = CredentialCache.DefaultCredentials;
    }

    /// <summary>
    /// On each 'Go' button click initialize connection because settings might be changed
    /// </summary>
    private void InitializeConnection()
    {
      #if DEBUG
      logIt(String.Format("{0} fired", (new StackTrace(true).GetFrame(0).GetMethod().Name)));
      #endif
      InitializeProxy();
      cli.Headers.Add("user-agnet", DictUtils.GetAppSetting("userAgent"));
      if (!_userDefinedProxy.pIsUserDefined) { return; } // no user defined proxy - user system settings
      cli.Headers.Add("Proxy-Authorization",
                      string.Format("Basic {0}",
                                    Convert.ToBase64String(Encoding.ASCII.GetBytes(String.Format("{0}:{1}",
                                                                                                 _userDefinedProxy.pUser, _userDefinedProxy.pPass)))));
    }

    /// <summary>
    /// prepare the proxy (if any) to work with
    /// </summary>
    private void InitializeProxy()
    {
      #if DEBUG
      logIt(String.Format("{0} fired", (new StackTrace(true).GetFrame(0).GetMethod().Name)));
      #endif
      if (DictUtils.GetAppSetting("useSysProxy", "false").Equals("true", StringComparison.InvariantCultureIgnoreCase))
      {
        cli.Proxy = WebRequest.GetSystemWebProxy();// :  ProxySetUp();
        cli.Proxy.Credentials = CredentialCache.DefaultCredentials;
        #if DEBUG
        logIt("Proxy set to System Proxy");
        #endif
        return;
      }
      cli.Proxy = ProxySetUp();
      cli.UseDefaultCredentials = false;
      cli.Credentials = cli.Proxy.Credentials;
      #if DEBUG
      logIt(String.Format("Will user defined proxy [{0}:{1}].", _userDefinedProxy.pUrl, _userDefinedProxy.pPort));
      #endif
    }

    private IWebProxy ProxySetUp()
    {
      #if DEBUG
      logIt(String.Format("{0} fired", (new StackTrace(true).GetFrame(0).GetMethod().Name)));
      logIt(String.Format("Trying to set proxy to {0}", DictUtils.GetAppSetting("defaultProxy")));
      #endif
      if(String.IsNullOrEmpty(_userDefinedProxy.pUrl)){
        #if DEBUG
        logIt(String.Format("No valid proxy found [{0}]", _userDefinedProxy.pUrl));
        #endif
        return new WebProxy() as IWebProxy;
      }
      WebProxy p = null;
      try {  p = new WebProxy(_userDefinedProxy.pUrl, _userDefinedProxy.pPort); }
      catch (Exception ex) {
        MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
        return new WebProxy() as IWebProxy;
      }
      p.UseDefaultCredentials = false;
      p.Credentials = new NetworkCredential(_userDefinedProxy.pUser, _userDefinedProxy.pPass);
      return p as IWebProxy;
    }

    #endregion

    #region [ComboBox methods]

    /// <summary>
    /// When ENTER is pressed do next step
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    void Cb_KeyDown(object sender, KeyEventArgs e)
    {
      ComboBox _cb = sender as ComboBox;
      switch (_cb.Name)
      {
        case "cbPath":
          if (e.KeyCode == Keys.Return)
          {
            if (cbPath.Items.Count == 0 || String.IsNullOrEmpty(cbPath.Text)) { btnFolder.PerformClick(); }
            cbWordsList.Focus();
          }
          break;
        case "cbWordsList":
          if (e.KeyCode == Keys.Return) { btnGo.PerformClick(); }
          break;
      }
    }

    
    /// <summary>
    /// When a text box has the focus - select all contents.
    /// </summary>
    private void cbEntered(object sender, EventArgs e)    {      (sender as ComboBox).SelectAll();    }

    /// <summary>
    /// Event handler for text boxes when foculs has been lost.
    /// </summary>
    /// <param name="sender"><seealso cref="System.Windows.Forms.TextBox"/> control fired event.</param>
    /// <param name="e"><seealso cref="System.EventArgs"/> parameters for this event.</param>
    private void userPathLeave(object sender, EventArgs e)
    {
      ComboBox cb = sender as ComboBox;
      if(String.IsNullOrEmpty(cb.Text)) {  return; } // if path is empty do not check
      if (!hasWriteAccess(cb.Text))
      {
        logIt(String.Format("No write access to {0}.", cb.Text), LogTextType.Error);
        MessageBox.Show(String.Format("[{0}] is not accessible!", cb.Text));
        cb.Focus();
        cb.SelectAll();
        return;
      }
#if DEBUG
      logIt(String.Format("write access to {0} - OK.", cb.Text));
#endif
      if(DictUtils.GetAppSetting("cbPathHistoryAllowed", "true").ToLower().Equals("false")) {  return;  } // history disabled
      AddToPathHistory(); // if history allowed - add text to
      InitializeWords();    // on each new path build words list
    }

    /// <summary>
    /// add currrent words to history, if it is allowed
    /// </summary>
    void CbWordsList_Leave(object sender, EventArgs e)
    {
      ComboBox _cb = sender as ComboBox;
      _cb.Text = _cb.Text.Trim().ToLower();
      if(String.IsNullOrEmpty(_cb.Text)) { return; } /* there is no word to search for */
      if(cbWordsList.DataSource == null) { return; } /* word list is not built yet */
      if((cbWordsList.DataSource as List<String>).IndexOf(_cb.Text) > -1) { return; } /* word exists in the list */
//      (cbWordsList.DataSource as List<String>).Insert(0, _cb.Text); /* when word does not exist in the list - add it */
      return;
    }
    #endregion
    
    #region [Outer Processes]
    
    void RtbLog_LinkClicked(object sender, LinkClickedEventArgs e)
    {
      System.Diagnostics.Process.Start(e.LinkText);
      cbWordsList.Focus();
      cbWordsList.SelectAll();
    }
    
    #endregion

    #region [Web Client handlers]

    private void webClient_ProgressChanged(object sender, DownloadProgressChangedEventArgs e)
    {
      toolStripProgressBar1.Value = e.ProgressPercentage;
    }

    /// <summary>
    /// event handler for WebClient DownloadDataCompleted event;
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void WebClient_DownloadDataCompleted(object sender, DownloadDataCompletedEventArgs e)
    {
      String searchableWord;
      #if DEBUG
      logIt(String.Format("{0} fired", (new StackTrace(true).GetFrame(0).GetMethod().Name)));
      #endif
      String _userState = e.UserState as string;
      String word = _userState.Contains("@") ? _userState.Split(new char[] {'@'})[0] : _userState;
      try  {  searchableWord = Regex.Replace(word, @"[\s\-]{1,}", "_");    }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
        logIt(String.Format("{0}: {1}", (new StackTrace(true).GetFrame(0).GetMethod().Name), ex.Message));
        return;
      }
      if (e.Cancelled)
      {
        logIt(String.Format("{0}: Operation canceled.", (new StackTrace(true).GetFrame(0).GetMethod().Name)), LogTextType.Error);
        UpdateStatus(String.Format("Requiest for [{0}] canceled.", word));
        ResetButtons();
        return;
      }
      String currentPath = GetPath();
      if (e.Error != null)
      {
        logIt(String.Format("Error: {0}", e.Error.Message), LogTextType.Error);
        UpdateStatus(e.Error.Message);
        ResetButtons();
        return;
      }
      #if DEBUG
      logIt(String.Format("Searching for {0} completed. Getting result . . .", word));
      #endif
      byte[] myDataBuffer = (byte[])e.Result;

      string contents = Encoding.ASCII.GetString(myDataBuffer);
      #if DEBUG
      logIt(String.Format("Result got. Searching for {0}*.mp3 sound file . . .", word));
      #endif
      Regex re = new Regex(String.Format(@"http:\/\/[\w\._\/]+{0}[\w\._\/]+mp3", searchableWord));
      MatchCollection mc = re.Matches(contents.Replace(Environment.NewLine, String.Empty));
      if (mc.Count == 0)
      {
        string url = _userState.Contains("@") ? String.Format(" at {0}", _userState.Split(new char[] {'@'})[1]) : String.Empty;
        logIt(String.Format("[{0}] not found{1}.", word, url), LogTextType.Error);
        UpdateStatus(String.Format("[{0}] not found.", word));
        GetWord();
        return;
      }
      #if DEBUG
      logIt(String.Format("[{0}] found. Trying to download . . .", word));
      #endif
      UpdateStatus(String.Format("[{0}] found. Trying to download . . .", word));
      String fileName = MakeFileNameValid(String.Format("{0}{1}.mp3", currentPath, Regex.Replace(word, @"\s{1,}", "_")));
      try
      {
        toolStripProgressBar1.Value = 0;
        toolStripProgressBar1.Visible = true;
        #if DEBUG
        logIt(String.Format("Sucess. Going to save it to {0} . . .", fileName));
        #endif
        if (!this.hasWriteAccess(cbPath.Text)) { throw new IOException(String.Format("No write access to {0}.", cbPath.Text));   }  // check write permission before downloading file
        if(File.Exists(fileName)) 
        {
          SuccessWordDownload(word);
          return;
        }
        cli.DownloadFileAsync(new Uri(mc[0].Value), fileName, fileName);
      }
      catch (Exception ex)
      {
        toolStripProgressBar1.Visible = false;
        MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
        #if DEBUG
        logIt(String.Format("Error! {0}", ex.Message));
        #endif
        return;
      }
      #if DEBUG
      logIt("Download started . . .");
      #endif
    }

    /// <summary>
    /// event handler for WebClient DownloadFileCompleted event;
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void WebClient_DownloadFileCompleted(object sender, AsyncCompletedEventArgs e)
    {
      #if DEBUG
      logIt(String.Format("{0} fired", (new StackTrace(true).GetFrame(0).GetMethod().Name)));
      #endif
      if(e.Error != null)
      {
        logIt(e.Error.Message, LogTextType.Error);
        goto NextWord;
      }
      string resWord = e.UserState as string;
      SuccessWordDownload(resWord);
      #if DEBUG
      logIt("Checking for more words to search for . . .");
      #endif
    NextWord:
      GetWord();
    }

/// <summary>
/// Save path where word is downloaded and log result status
/// </summary>
/// <param name="resWord">word that has been donwloaded</param>
    private void SuccessWordDownload(String resWord)
    {
      logIt(String.Format("Result saved at {0}", resWord), LogTextType.Sucess);
      UpdateStatus(String.Format("Result saved at {0}", resWord));
      try
      {
        Clipboard.SetText(resWord);
        InitializeWords(); // regenerate word list
        #if DEBUG
        logIt(String.Format("File location [{0}] saved in the clipboard.", resWord));
        #endif
      }
      catch (Exception ex)
      {
        logIt(ex.Message);
        logIt(ex.InnerException.Message);
        UpdateStatus(ex.Message);
      }
    }
    
    #endregion

    #region [Form methods]
    /// <summary>
    /// Save configuration setting on exit. If file does not exist will be created.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void dict_FormClosing(object sender, FormClosingEventArgs e)
    {
      DictUtils.SaveSettings();
    }
    #endregion

    #region [Button methods]

    /// <summary>
    /// Button opening the dialig for folder choice
    /// </summary>
    /// <param name="sender">the button clicked</param>
    /// <param name="e">event parameters - in the case - null</param>
    private void btnFolder_Click(object sender, EventArgs e)
    {
      #if DEBUG
      logIt(String.Format("{0} fired", (new StackTrace(true).GetFrame(0).GetMethod().Name)));
      #endif
      //      if (!String.IsNullOrEmpty(tbPath.Text))
      if (!String.IsNullOrEmpty(cbPath.Text))
      {
        folderBrowserDialog1.SelectedPath = cbPath.Text;
      }
      if(folderBrowserDialog1.ShowDialog() == DialogResult.Cancel) { return;  }
      if(String.IsNullOrEmpty(folderBrowserDialog1.SelectedPath)) { return;  } /* no path selected */
      cbPath.Text = folderBrowserDialog1.SelectedPath;
      if (cbPath.Items.Count == 0 || cbPath.Text.Length > 0)  // if folder has been chosen go to next field
      {
        cbWordsList.Focus();
      }
      else { cbPath.Focus(); }  // else return back
      InitializeWords(); // build a new list from the new path
      AddToPathHistory(); // if history allowed - add text to
    }

    /// <summary>
    /// Start searching
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void btnGo_Click(object sender, EventArgs e)
    {
      #if DEBUG
      logIt(String.Format("{0} fired", (new StackTrace(true).GetFrame(0).GetMethod().Name)));
      #endif
      btnStop.Enabled = true;
      rtbLinesCounter = 0;
      rtbLog.Clear();
      if (String.IsNullOrEmpty(cbWordsList.Text))
      {
        MessageBox.Show("Please enter a word to search.");
        return;
      }
      try  {  userWords = Regex.Replace(cbWordsList.Text, @"\s{0,},\s{0,}", ",").Split(new char[] { ',' }).ToList<String>();  }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
        return;
      }
      #if DEBUG
      logIt("Trying to initiate connetion.");
      logIt("Trying to GetWord...");
      #endif
      InitializeConnection();
      GetWord();
      #if DEBUG
      logIt("GetWord returned");
      #endif
      cbWordsList.Focus();
      cbWordsList.SelectAll();
    }

    /// <summary>
    /// Stops current search
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void btnStop_Click(object sender, EventArgs e)
    {
      ResetButtons();
      cli.CancelAsync();
    }

    private void btnSettings_MouseHover(object sender, EventArgs e)
    {
      btnSettings.Image = DictNS.Properties.Resources.settings12_active;
    }

    private void btnSettings_MouseLeave(object sender, EventArgs e)
    {
      btnSettings.Image = DictNS.Properties.Resources.settings12_inactive;
    }

    private void btnSettings_Click(object sender, EventArgs e)
    {
      String _tag = Convert.ToString((sender as Button).Tag);
      frmSettings _settings = null;
      if (String.IsNullOrEmpty(_tag))   {  _settings = new frmSettings(); }
      else
      {
        _settings = new frmSettings("se");
        (sender as Button).Tag = null;
      }
      if (!(_settings is frmSettings))
      {
        MessageBox.Show("Failed to open Settings dialog", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
        return;
      }
      _settings.StartPosition = FormStartPosition.CenterParent;
      try
      {
        _settings.ShowDialog(this);
        if (!_settings.IsDisposed) { _settings.Dispose(); }
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
        return;
      }
    }

    #endregion

    #region [Helper methods]

    /// <summary>
    /// Start seach process
    /// </summary>
    private void GetWord()
    {
      #if DEBUG
      logIt(String.Format("method [{0}] fired", (new StackTrace(true).GetFrame(0).GetMethod().Name)));
      #endif
      String currentPath = GetPath(); // choose where to store result
      String se = DictUtils.GetDefaultSearchEngine();
      if (String.IsNullOrEmpty(se))
      {
        btnSettings.Tag = "se";
        btnSettings.PerformClick();
        se = DictUtils.GetDefaultSearchEngine();
        if (String.IsNullOrEmpty(se)) {  return;  }
      }
      #if DEBUG
      logIt(String.Format("Path to store data: [{0}].", currentPath));
      logIt(String.Format("Words found: [{0}]{1}.", userWords.Count(), (userWords.Count() > 0 ? " - continue." : " - end of search.")));
      #endif
      btnGo.Enabled = userWords.Count == 0;
      if (userWords.Count == 0)
      {
        toolStripProgressBar1.Visible = false;
        btnStop.Enabled = false;
        return;
      }
      String word = userWords[0]; // extract left most word in the list
      userWords.RemoveAt(0); // delete it
      #if DEBUG
      logIt(String.Format("Searching for: [{0}].", word));
      #endif
      String fileName = String.Format("{0}{1}.mp3", currentPath, Regex.Replace(word, @"\s{1,}", "_"));
      if (File.Exists(fileName))
      {
        if (MessageBox.Show(String.Format("{0} exists. Overwrite it?.", fileName), AppDomain.CurrentDomain.FriendlyName, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.No)
        {
          Clipboard.SetText(fileName);  // put file path in clipboard anyway
          logIt(String.Format("{0} exists. Overrite canceled.", fileName));
          UpdateStatus(String.Format("{0} exists. Overrite canceled.", fileName));
          ResetButtons();
          return;
        }
        try { File.Delete(fileName); } 
        catch (Exception x) 
        {
          toolStripProgressBar1.Visible = false;
          MessageBox.Show(x.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
  #if DEBUG
          logIt(String.Format("Error! {0}", x.Message));
  #endif
          return;
        }
      }
      UpdateStatus(String.Format("Searching for [{0}] . . .", word));
      try           // if this is direct sound file - download it
      {
        toolStripProgressBar1.Value = 0;
        toolStripProgressBar1.Visible = true;
        if (se.EndsWith("mp3"))
        {
          String w = Regex.Replace(word, @"\s{1,}", "_");
          String url = se.Replace("%s", w);
          Uri uri = new Uri(url);
          String userParam = String.Format("{0}{1}.mp3", currentPath, w, url);
          cli.DownloadFileAsync(uri, userParam, userParam);
        }
        else
        {
          String url  = se.Replace("%s", Regex.Replace(word, @"\s{1,}", "-"));
          cli.DownloadDataAsync(new Uri(url), String.Format("{0}@{1}", word, url));
        }
      }
      catch (Exception ex)
      {
        toolStripProgressBar1.Visible = false;
        MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
#if DEBUG
        logIt(String.Format("Error! {0}", ex.Message));
#endif
        return;
      }
#if DEBUG
      logIt("Download started.");
#endif
    }

    /// <summary>
    /// return full path where result will be stored
    /// </summary>
    /// <returns>full path ending with a backslash</returns>
    private string GetPath()
    {
      string currentPath = (String.IsNullOrEmpty(cbPath.Text) ? Directory.GetCurrentDirectory() : cbPath.Text);
      //      string currentPath = (String.IsNullOrEmpty(tbPath.Text) ? Directory.GetCurrentDirectory() : tbPath.Text);
      currentPath += !currentPath.EndsWith("\\") ? "\\" : String.Empty;
      return currentPath;
    }

    /// <summary>
    /// Check word for OS reserved keywords and adds one (1) at the end it it is
    /// </summary>
    /// <param name="fileName">Full path where file have to be saved</param>
    /// <returns>Full path with corrected filename</returns>
    private String MakeFileNameValid(String fileName)
    {
      String[] _keyWords = new String[] { "con", "prn", "aux", "clock$", "nul", 
                                          "com0", "com1", "com2", "com3", "com4",
                                          "com5", "com6", "com7", "com8", "com9", 
                                          "lpt0", "lpt1", "lpt2", "lpt3", "lpt4",
                                          "lpt5", "lpt6", "lpt7", "lpt8", "lpt9" };
      String fileWithoutExt = Path.GetFileNameWithoutExtension(fileName);
      if(Array.IndexOf(_keyWords, fileWithoutExt) > -1) {
        fileName = Regex.Replace(fileName, fileWithoutExt, fileWithoutExt+"1");
      }
      return fileName;
    }
    
    /// <summary>
    /// Check for write permissions of the chosen folder
    /// </summary>
    /// <param name="_path"><seealso cref="System.String"/> with the path user has been entered.</param>
    /// <returns>true if can wirte; false - if cannot</returns>
    private bool hasWriteAccess(string _path)
    {
      PermissionSet perm;
      if (!Directory.Exists(_path))
      {
        try  {     Directory.CreateDirectory(_path);    }
        catch (Exception e)
        {
          DictUtils._dump(e);
          logIt(String.Format(String.Format("[{0}] {1}", _path, e.Message)), LogTextType.Error);
          UpdateStatus(String.Format("[{0}] {1}", _path, e.Message));
          return false;
        }
      }
      try
      {
        perm = new PermissionSet(PermissionState.None);
        perm.AddPermission(new FileIOPermission(FileIOPermissionAccess.Write, _path));
      }
      catch (Exception e)
      {
        DictUtils._dump(e);
        MessageBox.Show(e.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
        UpdateStatus(String.Format("[{0}] {1}", _path, e.Message));
        return false;
      }
      return perm.IsSubsetOf(AppDomain.CurrentDomain.PermissionSet);
    }

    /// <summary>
    /// show result from the actions during search and retrieve process
    /// </summary>
    /// <param name="msg"><seealso cref="System.String"/> with the text that will be shown</param>
    /// <param name="_type"><seealso cref="LogTextType"/> enum with possible result values</param>
    private void logIt(String msg, LogTextType _type = LogTextType.Info)
    {
      rtbLog.AppendText(String.Format("{0:000}. {1}", (++rtbLinesCounter), msg.Trim()));
      rtbLog.ScrollToCaret();
      rtbLog.Select(rtbLog.Text.LastIndexOf(rtbLog.Lines[rtbLog.Lines.Length - 1]),
                    rtbLog.Lines[rtbLog.Lines.Length - 1].Length); // select newly created line
      switch (_type)
      {
          case LogTextType.Sucess: rtbLog.SelectionColor = Color.Green; break;
          case LogTextType.Error: rtbLog.SelectionColor = Color.Red; break;
          default: rtbLog.SelectionColor = Color.DimGray; break;
      }
      rtbLog.AppendText(Environment.NewLine);
    }

    /// <summary>
    /// Show message in the statsu bar for the user.
    /// </summary>
    /// <param name="s">Message to show.</param>
    private void UpdateStatus(String s)
    {
      toolStripStatusLabel1.Text = s;
      statusStrip1.Refresh();
    }

    /// <summary>
    /// Restore initial state of the buttons
    /// </summary>
    private void ResetButtons()
    {
      btnStop.Enabled = false;
      btnGo.Enabled = true;
    }

    /// <summary>
    /// Prepare user defined proxy data
    /// </summary>
    private void GetUserProxy()
    {
      _userDefinedProxy.pIsUserDefined = DictUtils.GetAppSetting("useSysProxy").Equals("false", StringComparison.InvariantCultureIgnoreCase);
      if (!_userDefinedProxy.pIsUserDefined) { return;  }
      String[] _proxy = DictUtils.GetAppSetting("userProxy").Split('|');
      _userDefinedProxy.pUrl = _proxy[0];
      //_userDefinedProxy.pUrl = String.Format("{0}://{1}", (String.IsNullOrEmpty(_proxy[4]) ? "http" : _proxy[4]), _proxy[0]);
      if (!String.IsNullOrEmpty(_proxy[1])) { _userDefinedProxy.pPort = Convert.ToInt32(_proxy[1]); }
      _userDefinedProxy.pUser = _proxy[2];
      _userDefinedProxy.pPass = _proxy[3];
    }
    
    /// <summary>
    /// Add path entered by user if history enabled
    /// </summary>
    /// <param name="cb"></param>
    private void AddToPathHistory()
    {
    	if(String.IsNullOrEmpty(cbPath.Text)) { return;  }
    	string cbPathCurrentValue = cbPath.Text;
      int maxNumPaths = -1;
      Int32.TryParse(DictUtils.GetAppSetting("nudNumPaths", "9"), out maxNumPaths);
      bool isNewPathAdded = false;
      List<String> pathList = cbPath.DataSource as List<String>;
      if (pathList == null) {  pathList = new List<string>(maxNumPaths); }
    
      if (cbPath.Items.IndexOf(cbPathCurrentValue) == -1) {		// if text does not exists into collection
#if DEBUG
logIt(String.Format("{0} not found in the list.", cbPath.Text));
#endif
        cbPath.DataSource = null;
        if (pathList != null && pathList.Count == maxNumPaths) {
          pathList.RemoveAt(pathList.Count - 1);      // remove last if exceeds max number of paths
#if DEBUG
logIt(String.Format("Last entry removed from history because max entries allowed {0} has been hit.", maxNumPaths.ToString()));
#endif
        }
        pathList.Insert(0, cbPathCurrentValue);	//  add the new path first in the list
#if DEBUG
logIt("Path inserted at position 0.");
#endif
        cbPath.DataSource = pathList; // bind it back
        cbPath.SelectedIndex = 0;
        isNewPathAdded = true;
      }
      if (cbPath.SelectedIndex < 0) {
        cbPath.SelectedIndex = cbPath.Items.IndexOf(cbPathCurrentValue);
      }
      DictUtils.SetAppSetting("activeUserPath", cbPath.SelectedIndex.ToString());
      if (isNewPathAdded) {
      	DictUtils.SetAppSetting("userPath", ";" + Regex.Replace(String.Join(";", pathList.ToArray()), @";{2,}", ";"));
      }
#if DEBUG
logIt("Path history updated.");
#endif
    }

    #endregion

  }
}
